function love.load()
	width = love.graphics.getWidth()
	height= love.graphics.getHeight()
	music = love.audio.newSource("Renich_-_To_be_happy_extract.ogg")
	mandel = love.graphics.newPixelEffect [[
		extern float w;
		extern float h;
		extern float max_it;
		extern float sx; // scale x
		extern float sy; // scale y
		extern float px; // view x
		extern float py; // view y
		extern float dt;
		extern float dt2;
		float infinity(float a, float b)
		{
			float x = 0;
			float y = 0;
			float x2 = 0;
			float y2 = 0;
			float i = max_it;
			while ((x*x*dt + y*y < 4) && (i > -1))
			{
				x2 = x*x - y*y + a + x*dt + y*dt2;
				y = 2*x*y + b;
				x = x2;
				i = i - 1;
			}
			return 1-(max_it-i)/(max_it);
		}
		vec4 effect(vec4 co, Image tex, vec2 tc, vec2 pc)
		{
			float col = infinity( px - sx/2 + sx*pc.x/w, py - sy/2 + sy*pc.y/h);
			vec4 initColor = Texel(tex, tc);
			return vec4(col-initColor.r,col-initColor.g,col-initColor.b,1);
		}
	]]
	love.graphics.setBackgroundColor(255,255,255)
	love.graphics.setPixelEffect(mandel)
	logo = love.graphics.newImage("logo.png")
	text = love.graphics.newImage("text.png")

	margin = 0.3 -- for mouse control
	
	-- initial pos
	pos0 = { zoom = 1.4, x = -2, y = 0 }
	
	-- middle pos
	pos1 = { time = 2.13, zoom = 1.1, x = -1.15, y = 0 }
	
	-- final pos
	pos2 = { time = 8, zoom = 0.6, x = 1.25, y = 0 }
	
	-- current pos
	pos = {zoom = pos0.zoom, x = pos0.x, y = pos0.y}
	max_it = 12
	time = 0
	time2 = 0
	animate = true;
	drawtext = true;
	love.audio.play(music)
	music:setLooping(true)
end

function love.draw()
	mandel:send("w", width)
	mandel:send("h", height)
	mandel:send("max_it", max_it)
	mandel:send("px", pos.x)
	mandel:send("py", pos.y)
	mandel:send("sx", 1/math.sqr(pos.zoom))
	mandel:send("sy", 1/math.sqr(pos.zoom))
	mandel:send("dt", math.sin(time)/10)
	mandel:send("dt2",math.sin(time2)/10)
	love.graphics.setColor(255,255,255)
	love.graphics.rectangle("fill", 0, 0, width, height)
	if(drawtext) then
		if(time > 3.7) then
			love.graphics.draw(logo, width/2-256, height/2-128)
		end
		if(time > 10) then
			love.graphics.draw(text,0,height-16)
		end
	end
end

function love.update(dt)
	if(animate) then
		time = time + dt
		time2 = time2 + dt*1.3567
	end
	if(time<=pos1.time) then
		proptime = time / pos1.time
		pos.x = pos0.x + (pos1.x - pos0.x)*proptime
		pos.zoom = pos0.zoom + (pos1.zoom-pos0.zoom)*proptime
	elseif(time<=pos2.time) then
		proptime = (time-pos1.time) / (pos2.time-pos1.time)
		pos.x = pos1.x + (pos2.x - pos1.x)*proptime
		pos.zoom = pos1.zoom + (pos2.zoom-pos1.zoom)*proptime
	elseif(time > 10) then
		if love.mouse.isDown("l") then
			pos.zoom = pos.zoom + pos.zoom*dt
		end
		if love.mouse.isDown("r") then
			pos.zoom = pos.zoom - pos.zoom*dt
		end
		propx = love.mouse.getX()/width
		propy = love.mouse.getY()/height
		if propx > (1-margin) then
			pos.x = pos.x + math.sqr(5*(propx - (1-margin)))*dt/(pos.zoom*pos.zoom)
		end
		if propx < margin then
			pos.x = pos.x - math.sqr(5*(margin - propx))*dt/(pos.zoom*pos.zoom)
		end
		if propy > (1-margin) then
			pos.y = pos.y - math.sqr(5*(propy - (1-margin)))*dt/(pos.zoom*pos.zoom)
		end
		if propy < margin then
			pos.y = pos.y + math.sqr(5*(margin - propy))*dt/(pos.zoom*pos.zoom)
		end
	end
end

function love.mousepressed(x,y,button)
	if(button == "wu") then
		max_it = max_it+4
	end
	if(button == "wd" and max_it > 4) then
		max_it = max_it-4
	end
end

function love.keypressed(key)
	if key == "escape" then
		love.event.push("quit")
	end
	if key == "return" then
		if animate then
			animate = false
		else
			animate = true
		end
	end
	if key == "h" then
		if drawtext then
			drawtext = false
		else
			drawtext = true
		end
	end
	if key == " " then
		print("============")
		print("time :\t"..time)
		print("zoom :\t"..pos.zoom)
		print("px :\t"..pos.x)
		print("py :\t"..pos.y)
		print("maxit:\t"..max_it)
	end
end

function math.sqr(x)
	return x*x
end
